unit nData;   { 1992.9.26  programmed by Masaaki Wada }



interface


uses
	str, picture;

const
	maxNumStrips = 200;
	blockSize = 100;
	pageWidth = 2000;
	pageHeight = 3200;

type
	direction = 1..9; { on numeric key pad,  5 = not specified }
	stripKind = (nKind, uKind, lKind, rKind, tKind, bKind);

	stripRecord = packed record
			kind: stripKind;
			index, width: byte;
			dir: direction;
			comp1, comp2: byte;
			temp1, temp2: byte
		end;

	nData = record
			error: boolean;
			errorMessage: string;
			numStrips: integer;
			strip: array[1..maxNumStrips] of stripRecord;
			num: array[stripKind] of integer;
			numComponents: integer
		end;

procedure nDataFromStr (var nd: nData; s: string);
function nDataToStr (nd: nData): string;
procedure makePicture (nd: nData);





implementation




procedure nDataFromStr (var nd: nData; s: string);


	procedure err (msg: string);
		begin
			nd.error := true;
			nd.errorMessage := msg;
			exit(nDataFromStr)
		end;


	procedure pushStrip (sr: stripRecord);
		begin
			with nd do
				begin
					numStrips := numStrips + 1;
					strip[numStrips] := sr;
					num[sr.kind] := num[sr.kind] + 1
				end
		end;


	procedure pass1; { set numStrips, kind, index, dir }

		var
			ss, sdir: string;
			knd: stripKind;
			count, i, stage: integer;
			sr: stripRecord;

		function getWord (var s: string): string;
			begin
				getWord := strGetW(s, all - [' ']);
				strSkip(s, [' '])
			end;

		function getCount (var s: string): integer;
			begin
				if strCIn(s, 1, numeral) then
					getCount := strGetLong(s)
				else
					getCount := 1
			end;

		function getKind (var s: string): stripKind;
			var
				c: char;
			begin
				c := strGetC(s);
				case c of
					'n': 
						getKind := nKind;
					'u': 
						getKind := uKind;
					'l': 
						getKind := lKind;
					'r': 
						getKind := rKind;
					't': 
						getKind := tKind;
					'b': 
						getKind := bKind;
					otherwise
						err(concat('unknown kind ''', c, ''''))
				end
			end;

		function getParen (var s: string): string;
			var
				l, r: integer;
			begin
				l := pos('(', s);
				r := pos(')', s);
				if l = 0 then
					getParen := ''
				else if r < l then
					err('nonmatching parentheses')
				else
					begin
						getParen := copy(s, l + 1, r - l - 1);
						delete(s, l, r - l + 1)
					end
			end;

		function getDirection (var s: string): direction;
			begin
				if s = '' then
					getDirection := 5
				else if strCIn(s, 1, ['u', 'd', 'l', 'r']) then
					case strGetC(s) of
						'u': 
							getDirection := 8;
						'd': 
							getDirection := 2;
						'l': 
							getDirection := 4;
						'r': 
							getDirection := 6
					end
				else
					err(concat('unknown direction (', s, ')'))
			end;


		begin { pass1 }
			strSkip(s, [' ']);
			if s = '' then
				err('string empty');

			with nd do
				begin
					numStrips := 0;
					for knd := nKind to bKind do
						num[knd] := 0;
					stage := 1;
					repeat
						ss := getWord(s);
						count := getCount(ss);
						sr.kind := getKind(ss);
						sr.index := strGetLong(ss);
						sdir := getParen(ss);

						case sr.kind of
							tKind: 
								begin
									if stage > 1 then
										err('t-kind must appear first')
									else
										stage := 2;
									if count <> 1 then
										err('count for t-kind must be 1');
									count := sr.index;
									for i := 1 to count do
										begin
											sr.index := i;
											sr.dir := getDirection(sdir);
											pushStrip(sr)
										end;
									if length(sdir) > 0 then
										err(concat('error in direction (', sdir, ')'))
								end;
							nKind, uKind, lKind, rKind: 
								begin
									if stage > 2 then
										err('b-kind must appear last')
									else
										stage := 2;
									if sr.index <= 0 then
										err('Index must be positive');
									sr.dir := getDirection(sdir);
									for i := 1 to count do
										pushStrip(sr)
								end;
							bKind: 
								begin
									stage := 3;
									if count <> 1 then
										err('count for b-kind must be 1');
									count := sr.index;
									sr.index := 1;
									for i := 1 to count do
										begin
											sr.dir := getDirection(sdir);
											pushStrip(sr)
										end;
									if length(sdir) > 0 then
										err(concat('error in direction (', sdir, ')'))
								end
						end

					until s = ''
				end
		end; { pass1 }


	procedure pass2;   { set width }

		var
			w, i: integer;
			sr: stripRecord;


		procedure errIndex;
			begin
				err('index out of range')
			end;

		begin
			w := 0;
			with nd do
				begin
					for i := 1 to numStrips do
						with strip[i] do
							if index <= 0 then
								errIndex
							else
								case kind of
									tKind: 
										begin
											if index <> w + 1 then
												errIndex;
											w := w + 1;
											width := w
										end;
									nKind: 
										begin
											if index > w + 1 then
												errIndex;
											w := w + 2;
											width := w
										end;
									rKind, lKind: 
										begin
											if index > w - 1 then
												errIndex;
											width := w
										end;
									uKind: 
										begin
											if index > w - 1 then
												errIndex;
											width := w;
											w := w - 2
										end;
									bKind: 
										begin
											if w < 1 then
												err('too many b''s');
											width := w;
											w := w - 1
										end;
								end;
					with sr do
						begin
							kind := bKind;
							index := 1;
							dir := 5
						end;
					while w > 0 do
						begin
							sr.width := w;
							pushStrip(sr);
							w := w - 1
						end
				end
		end;


	procedure pass3; { set numComponents, comp's, dir }

		var
			i: integer;
			closed: boolean;

		procedure errDir;
			begin
				err('inconsistent direction')
			end;

		procedure clearComps;
			var
				i: integer;
			begin
				with nd do
					for i := 1 to numStrips do
						with strip[i] do
							begin
								comp1 := 0;
								comp2 := 0
							end
			end;

		procedure trace (i, ind: integer; frwd: boolean);
			var
				done: boolean;
				ud: direction;

			function realDir (d: direction): direction;
				begin
					if frwd then
						realDir := d
					else
						realDir := 10 - d
				end;

			function dirConsistent (d1, d2: direction): boolean;
				begin
					case d1 of
						1: 
							dirConsistent := (d2 in [1, 2, 4]);
						3: 
							dirConsistent := (d2 in [3, 2, 6]);
						7: 
							dirConsistent := (d2 in [7, 4, 8]);
						9: 
							dirConsistent := (d2 in [9, 6, 8])
					end
				end;

			procedure down_t;
				begin
					with nd.strip[i] do
						begin
							if ind >= index then
								ind := ind + 1;
							i := i + 1
						end
				end;

			procedure down_n;
				begin
					with nd.strip[i] do
						begin
							if ind >= index then
								ind := ind + 2;
							i := i + 1
						end
				end;

			procedure down_u;
				begin
					with nd do
						with strip[i] do
							begin
								if ind = index then
									begin
										comp1 := numComponents;
										if dir = 5 then
											dir := realDir(6)
										else if dir <> realDir(6) then
											errDir;
										ind := ind + 1;
										ud := 8;
										i := i - 1
									end
								else if ind = index + 1 then
									begin
										comp1 := numComponents;
										if dir = 5 then
											dir := realDir(4)
										else if dir <> realDir(4) then
											errDir;
										ind := ind - 1;
										ud := 8;
										i := i - 1
									end
								else
									begin
										if ind > index + 1 then
											ind := ind - 2;
										i := i + 1
									end
							end
				end;

			procedure down_lr;
				begin
					with nd do
						with strip[i] do
							begin
								if ind = index then
									begin
										comp1 := numComponents;
										case dir of
											5: 
												dir := realDir(3);
											1, 9: 
												dir := (dir + realDir(3)) div 2;
											otherwise
												if not dirConsistent(realDir(3), dir) then
													errDir
										end;
										ind := ind + 1
									end
								else if ind = index + 1 then
									begin
										comp2 := numComponents;
										case dir of
											5: 
												dir := realDir(1);
											3, 7: 
												dir := (dir + realDir(1)) div 2;
											otherwise
												if not dirConsistent(realDir(1), dir) then
													errDir
										end;
										ind := ind - 1
									end;
								i := i + 1
							end
				end;

			procedure down_b;
				begin
					with nd do
						with strip[i] do
							begin
								if ind = index then
									begin
										if dir = 5 then
											dir := realDir(2)
										else if dir <> realDir(2) then
											errDir;
										comp1 := numComponents;
										done := true
									end
								else
									begin
										if ind > index then
											ind := ind - 1;
										i := i + 1
									end
							end
				end;

			procedure up_t;
				begin
					with nd do
						with strip[i] do
							begin
								if ind = index then
									begin
										if dir = 5 then
											dir := realDir(8)
										else if dir <> realDir(8) then
											errDir;
										comp1 := numComponents;
										done := true
									end
								else
									begin
										if ind > index then
											ind := ind - 1;
										i := i - 1
									end
							end
				end;

			procedure up_n;
				begin
					with nd do
						with strip[i] do
							begin
								if ind = index then
									begin
										comp1 := numComponents;
										if dir = 5 then
											dir := realDir(6)
										else if dir <> realDir(6) then
											errDir;
										ind := ind + 1;
										ud := 2;
										i := i + 1
									end
								else if ind = index + 1 then
									if comp1 <> 0 then
										begin
											closed := true;
											done := true
										end
									else
										begin
											comp1 := numComponents;
											if dir = 5 then
												dir := realDir(4)
											else if dir <> realDir(4) then
												errDir;
											ind := ind - 1;
											ud := 2;
											i := i + 1
										end
								else
									begin
										if ind > index + 1 then
											ind := ind - 2;
										i := i - 1
									end
							end
				end;

			procedure up_u;
				begin
					with nd.strip[i] do
						begin
							if ind >= index then
								ind := ind + 2;
							i := i - 1
						end
				end;

			procedure up_lr;
				begin
					with nd do
						with strip[i] do
							begin
								if ind = index then
									begin
										comp2 := numComponents;
										case dir of
											5: 
												dir := realDir(9);
											3, 7: 
												dir := (dir + realDir(9)) div 2;
											otherwise
												if not dirConsistent(realDir(9), dir) then
													errDir
										end;
										ind := ind + 1
									end
								else if ind = index + 1 then
									begin
										comp1 := numComponents;
										case dir of
											5: 
												dir := realDir(7);
											1, 9: 
												dir := (dir + realDir(7)) div 2;
											otherwise
												if not dirConsistent(realDir(7), dir) then
													errDir
										end;
										ind := ind - 1
									end;
								i := i - 1
							end
				end;

			begin { trace }
				closed := false;
				ud := 2;
				done := false;
				while not done do
					with nd.strip[i] do
						case ud of
							2: 
								case kind of
									tKind: 
										down_t;
									nKind: 
										down_n;
									uKind: 
										down_u;
									lKind, rKind: 
										down_lr;
									bKind: 
										down_b
								end;
							8: 
								case kind of
									tKind: 
										up_t;
									nKind: 
										up_n;
									uKind: 
										up_u;
									lKind, rKind: 
										up_lr
								end
						end
			end; { trace }

		begin { pass3 }
			with nd do
				begin
					numComponents := 0;
					clearComps;
					for i := 1 to numStrips do
						with strip[i] do
							if comp1 = 0 then
								begin
									numComponents := numComponents + 1;
									comp1 := numComponents;
									case kind of
										tKind: 
											begin
												if dir = 5 then
													dir := 2;
												trace(i + 1, index, (dir = 2))
											end;
										nKind: 
											begin
												if dir = 5 then
													dir := 4;
												trace(i + 1, index, (dir = 4));
												if not closed then
													trace(i + 1, index + 1, (dir = 6))
											end
									end
								end
				end
		end; { pass3 }



	begin
		pass1;
		pass2;
		pass3;
	end;




function nDataToStr (nd: nData): string;

	var
		s, s1: string;
		count, i: integer;

	function strip2str (i: integer): string;
		var
			c: char;
		begin
			with nd.strip[i] do
				begin
					case kind of
						nKind: 
							c := 'n';
						uKind: 
							c := 'u';
						lKind: 
							c := 'l';
						rKind: 
							c := 'r';
						tKind: 
							c := 't';
						bKind: 
							c := 'b'
					end;
					strip2str := concat(c, long2str(index))
				end
		end;

	procedure addStrip (count: integer; s1: string);
		begin
			if count = 1 then
				s := concat(s, s1, ' ')
			else if count > 1 then
				s := concat(s, long2str(count), s1, ' ')
		end;

	begin { nDataToStr }
		with nd do
			if error then
				nDataToStr := concat('Error (', errorMessage, ')')
			else
				begin
					s := '';
					if num[tKind] > 0 then
						begin
							s1 := concat('t', long2str(num[tKind]));
							addStrip(1, s1)
						end;
					s1 := '';
					count := 0;
					for i := num[tKind] + 1 to numStrips - num[bKind] do
						if strip2str(i) = s1 then
							count := count + 1
						else
							begin
								addStrip(count, s1);
								s1 := strip2str(i);
								count := 1
							end;
					addStrip(count, s1);
					if num[bKind] > 0 then
						begin
							s1 := concat('b', long2str(num[bKind]));
							addStrip(1, s1)
						end;
					nDataToStr := copy(s, 1, length(s) - 1)
				end
	end;




procedure makePicture (nd: nData);


	var
		stripPlus: array[1..maxNumStrips] of record
				left, right, it: integer
			end;
		lMax, rMax, lLast, rLast: integer;


	procedure pass1;  { determine lMax and rMax }

		var
			l, r, i: integer;

		procedure setStripPlus;
			begin
				with stripPlus[i] do
					begin
						left := l;
						right := r;
						it := l + nd.strip[i].index - 1
					end;
			end;

		begin
			with nd do
				begin
					l := 0;
					r := num[tKind] - 1;
					lMax := l;
					rMax := r;
					for i := num[tKind] + 1 to numStrips - num[bKind] do
						with strip[i] do
							begin
								case kind of
									nKind: 
										begin
											if index = width - 1 then
												r := r + 2
											else if index = 1 then
												l := l - 2
											else
												begin
													l := l - 1;
													r := r + 1
												end;
											setStripPlus;
											if l < lMax then
												lMax := l;
											if r > rMax then
												rMax := r;
										end;
									uKind: 
										begin
											setStripPlus;
											if index = 1 then
												l := l + 2
											else if index = width - 1 then
												r := r - 2
											else
												begin
													l := l + 1;
													r := r - 1
												end
										end;
									otherwise
										setStripPlus
								end
							end
				end;
			lLast := l;
			rLast := r
		end;


	procedure pass2; { drawing knot diagram }

		var
			w, h, i, j, k: integer;
			scale, wctr, w2, hctr, h2, v: extended;
			sDir: array[-maxNumStrips..maxNumStrips] of direction;

		procedure connect (j: integer; d: direction);
			var
				x1, x2, x3, x4: extended;
			begin
				if sDir[j] + d = 10 then
					begin
						x2 := 0.25 * (d - 2);
						x1 := -x2;
						realLine(j + x1, v + 0.25, j + x2, v - 0.25)
					end
				else
					begin
						x1 := 0.25 * (sDir[j] - 8);
						x2 := 0.11 * (sDir[j] - 8);
						x3 := 0.11 * (d - 2);
						x4 := 0.25 * (d - 2);
						realBezier(j + x1, v + 0.25, j + x2, v + 0.11, j + x3, v - 0.11, j + x4, v - 0.25);
					end;
			end;

		procedure draw_n (j: integer);
			begin
				realBezier(j + 0.25, v - 0.75, j + 0.39, v - 0.61, j + 0.61, v - 0.61, j + 0.75, v - 0.75);
				sDir[j] := 9;
				sDir[j + 1] := 7;
			end;

		procedure draw_u (j: integer);
			begin
				connect(j, 3);
				connect(j + 1, 1);
				realBezier(j + 0.25, v - 0.25, j + 0.39, v - 0.39, j + 0.61, v - 0.39, j + 0.75, v - 0.25);
			end;

		procedure draw_l (j: integer);
			begin
				connect(j, 3);
				connect(j + 1, 1);
				realLine(j + 0.25, v - 0.25, j + 0.75, v - 0.75);
				realLine(j + 0.75, v - 0.25, j + 0.6, v - 0.4);
				realLine(j + 0.4, v - 0.6, j + 0.25, v - 0.75);
				sDir[j] := 9;
				sDir[j + 1] := 7;
			end;

		procedure draw_r (j: integer);
			begin
				connect(j, 3);
				connect(j + 1, 1);
				realLine(j + 0.25, v - 0.25, j + 0.4, v - 0.4);
				realLine(j + 0.6, v - 0.6, j + 0.75, v - 0.75);
				realLine(j + 0.75, v - 0.25, j + 0.25, v - 0.75);
				sDir[j] := 9;
				sDir[j + 1] := 7;
			end;

		begin
			with nd do
				begin
					w := rMax - lMax + 2;
					h := numStrips - num[tKind] - num[bKind] + 1;
					if num[tKind] > 0 then
						h := h + 1;
					if num[bKind] > 0 then
						h := h + 1;
					scale := pageHeight / h;
					if pageWidth / w < scale then
						scale := pageWidth / w;
					wctr := (lMax + rMax) / 2.0;
					w2 := pageWidth / 2.0 / scale;
					hctr := (h - 1) / 2.0;
					h2 := pageHeight / 2.0 / scale;

					openPic(pageWidth, pageHeight);
					setScale(wctr - w2, wctr + w2, -hctr - h2, -hctr + h2);
					setWidth(0.1);

					v := 0.0;

					if num[tKind] > 0 then
						begin
							for j := 0 to num[tKind] - 1 do
								begin
									realLine(j, v, j, v - 0.75);
									sDir[j] := 8
								end;
							v := v - 1.0
						end;

					for i := num[tKind] + 1 to numStrips - num[bKind] do
						with strip[i] do
							with stripPlus[i] do
								begin
									case kind of
										nKind: 
											if it = right - 1 then
												begin
													for j := left to right - 2 do
														begin
															connect(j, 2);
															realLine(j, v - 0.25, j, v - 0.75);
															sDir[j] := 8
														end;
													draw_n(it)
												end
											else if it = left then
												begin
													draw_n(it);
													for j := left + 2 to right do
														begin
															connect(j, 2);
															realLine(j, v - 0.25, j, v - 0.75);
															sDir[j] := 8
														end
												end
											else
												begin
													for j := left to it - 1 do
														begin
															connect(j + 1, 1);
															realLine(j + 0.75, v - 0.25, j + 0.25, v - 0.75);
															sDir[j] := 9
														end;
													for j := right downto it + 2 do
														begin
															connect(j - 1, 3);
															realLine(j - 0.75, v - 0.25, j - 0.25, v - 0.75);
															sDir[j] := 7
														end;
													draw_n(it);
												end;

										uKind: 
											if it = left then
												begin
													draw_u(it);
													for j := left + 2 to right do
														begin
															connect(j, 2);
															realLine(j, v - 0.25, j, v - 0.75);
															sDir[j] := 8
														end
												end
											else if it = right - 1 then
												begin
													for j := left to right - 2 do
														begin
															connect(j, 2);
															realLine(j, v - 0.25, j, v - 0.75);
															sDir[j] := 8
														end;
													draw_u(it)
												end
											else
												begin
													draw_u(it);
													for j := it - 1 downto left do
														begin
															connect(j, 3);
															realLine(j + 0.25, v - 0.25, j + 0.75, v - 0.75);
															sDir[j + 1] := 7
														end;
													for j := it + 2 to right do
														begin
															connect(j, 1);
															realLine(j - 0.25, v - 0.25, j - 0.75, v - 0.75);
															sDir[j - 1] := 9
														end
												end;

										lKind, rKind: 
											begin
												for j := left to it - 1 do
													begin
														connect(j, 2);
														realLine(j, v - 0.25, j, v - 0.75);
														sDir[j] := 8
													end;
												for j := it + 2 to right do
													begin
														connect(j, 2);
														realLine(j, v - 0.25, j, v - 0.75);
														sDir[j] := 8
													end;
												case kind of
													lKind: 
														draw_l(it);
													rKind: 
														draw_r(it)
												end
											end
									end;
									v := v - 1.0
								end;

					for j := lLast to rLast do
						begin
							connect(j, 2);
							realLine(j, v - 0.25, j, v - 1.0);
						end;

					closePic;
				end
		end;

	begin { makePicture }
		pass1;
		pass2;
	end;



end.