unit rData;   { 1992.9.26  programmed by Masaaki Wada }



interface


uses
	str, picture, nData;

type
	rRecordKind = (genKnd, posKnd, negKnd, chkKnd, stpKnd);

	rRecord = record
			knd: rRecordKind;
			in1, in2, out: integer;
			prevGen: integer
		end;

	rData = record
			numRR: integer;
			rr: array[1..maxNumStrips] of rRecord;
			numT, numN, numB: integer;
			Ts, Ns, Bs: array[1..maxNumStrips] of integer;
			numV: integer
		end;


procedure nDataToRData (nd: nData; var rd: rData);
function rDataToStr (rd: rData): string;


implementation



procedure nDataToRData (nd: nData; var rd: rData);


	procedure setInsAndOuts;
		var
			lane: array[1..maxNumStrips] of integer;
			i, j: integer;
		begin
			with rd do
				begin
					numRR := 0;
					numT := 0;
					numN := 0;
					numB := 0;
					numV := 0;
					with nd do
						for i := 1 to numStrips do
							with strip[i] do
								case kind of
									tKind: 
										begin
											numV := numV + 1;
											numRR := numRR + 1;
											with rr[numRR] do
												begin
													knd := genKnd;
													out := numV
												end;
											numT := numT + 1;
											Ts[numT] := numV;
											for j := width downto index + 1 do
												lane[j] := lane[j - 1];
											lane[index] := numV
										end;
									nKind: 
										begin
											numV := numV + 1;
											numRR := numRR + 1;
											with rr[numRR] do
												begin
													knd := genKnd;
													out := numV
												end;
											numN := numN + 1;
											Ns[numN] := numV;
											for j := width downto index + 2 do
												lane[j] := lane[j - 2];
											lane[index] := numV;
											lane[index + 1] := numV
										end;
									rKind: 
										begin
											numV := numV + 1;
											numRR := numRR + 1;
											with rr[numRR] do
												begin
													case dir of
														2, 4: 
															knd := negKnd;
														6, 8: 
															knd := posKnd
													end;
													in1 := lane[index + 1];
													in2 := lane[index];
													out := numV
												end;
											lane[index] := lane[index + 1];
											lane[index + 1] := numV
										end;
									lKind: 
										begin
											numV := numV + 1;
											numRR := numRR + 1;
											with rr[numRR] do
												begin
													case dir of
														2, 6: 
															knd := posKnd;
														4, 8: 
															knd := negKnd
													end;
													in1 := lane[index];
													in2 := lane[index + 1];
													out := numV
												end;
											lane[index + 1] := lane[index];
											lane[index] := numV
										end;
									uKind: 
										begin
											numRR := numRR + 1;
											with rr[numRR] do
												begin
													knd := chkKnd;
													case dir of
														6: 
															begin
																in1 := lane[index];
																in2 := lane[index + 1]
															end;
														4: 
															begin
																in1 := lane[index + 1];
																in2 := lane[index]
															end;
													end
												end;
											for j := index to width - 2 do
												lane[j] := lane[j + 2]
										end;
									bKind: 
										begin
											numB := numB + 1;
											Bs[numB] := lane[index];
											for j := index to width - 1 do
												lane[j] := lane[j + 1]
										end
								end
				end
		end;


	procedure optimizeRData;

		var
			noMoreChange: boolean;
			i, j: integer;
			rrec: rRecord;

		function rank (k: rRecordKind): integer;
			begin
				case k of
					genKnd: 
						rank := 3;
					posKnd, negKnd: 
						rank := 2;
					chkKnd: 
						rank := 1;
					stpKnd: 
						rank := 4
				end
			end;

		function raisable: boolean;
			begin
				if rrec.knd = genKnd then
					raisable := false
				else
					with rd do
						with rr[j - 1] do
							if rank(knd) < rank(rrec.knd) then
								raisable := false
							else if out = rrec.in1 then
								raisable := false
							else if out = rrec.in2 then
								raisable := false
							else
								raisable := true
			end;

		begin { optimizeRData }
			repeat
				noMoreChange := true;
				with rd do
					for i := 1 to numRR do
						begin
							rrec := rr[i];
							j := i;
							while raisable do
								begin
									if rank(rr[j - 1].knd) > rank(rrec.knd) then
										noMoreChange := false;
									rr[j] := rr[j - 1];
									j := j - 1
								end;
							rr[j] := rrec
						end
			until noMoreChange
		end;


	procedure putStop;
		begin
			with rd do
				begin
					numRR := numRR + 1;
					rr[numRR].knd := stpKnd
				end
		end;


	procedure setPrevGen;
		var
			prv, i: integer;
		begin
			prv := 0;
			with rd do
				for i := 1 to numRR do
					with rr[i] do
						begin
							prevGen := prv;
							if knd = genKnd then
								prv := i
						end
		end;


	begin { nDataToRData }
		setInsAndOuts;
		optimizeRData;
		putStop;
		setPrevGen
	end;



function rDataToStr (rd: rData): string;
	var
		s: string;
		i: integer;
	begin
		s := '';
		with rd do
			for i := 1 to numRR do
				with rr[i] do
					case knd of
						genKnd: 
							s := concat(s, long2str(out), ';');
						posKnd: 
							s := concat(s, long2str(in1), '`', long2str(in2), '=', long2str(out), ';');
						negKnd: 
							s := concat(s, '-', long2str(in1), '`', long2str(in2), '=', long2str(out), ';');
						chkKnd: 
							s := concat(s, long2str(in1), '?', long2str(in2), ';');
						stpKnd: 
							s[length(s)] := '.'
					end;
		rDataToStr := s
	end;


end.