unit slList; { 1993.6.21 by Masaaki Wada }


interface


uses
	small_gl2p;


const
	not_in_list = -1;
	not_defined = -1;


type
	itemHdl = ^itemPtr;
	itemPtr = ^itemRecord;
	itemRecord = record
			matrix: gl;
			mark: integer
		end;

	longHdl = ^longPtr;
	longPtr = ^longint;


	slList = object
			numOfItems: longint;
			itsItems: itemHdl;
			itsIndices: longHdl;

			procedure init (p: integer);
			procedure free;

			procedure append (g: sl);
			function getMat (i: longint): sl;
			function getMark (i: longint): integer;
			procedure setMark (i: longint; m: integer);
			function index (g: gl): longint;
			procedure listSLs;

			procedure show;
		end;




implementation


const
	lBlkSize = 1000;
	itemRecSize = 6;


type
	slPtr = ^sl;
	intPtr = ^integer;


{ auxiliary functions }

function matrixP (items: itemHdl; i: longint): slPtr;
	begin
		matrixP := slPtr(longint(items^) + itemRecSize * i)
	end;


function markP (items: itemHdl; i: longint): intPtr;
	begin
		markP := intPtr(longint(items^) + itemRecSize * i + 4)
	end;


function indexP (indices: longHdl; i: longint): longPtr;
	begin
		indexP := longPtr(longint(indices^) + i * 4)
	end;



{ methods }

procedure slList.init (p: integer);
	var
		items: itemHdl;
		indices: longHdl;
		longp, indexSize, i: longint;
	begin
		numOfItems := 0;

		items := itemHdl(NewHandle(lBlkSize * itemRecSize));
		itsItems := items;

		longp := p;
		indexSize := 2 * longp * longp * longp;
		indices := longHdl(NewHandle(indexSize * 4));
		itsIndices := indices;
		for i := 0 to indexSize - 1 do
			indexP(itsIndices, i)^ := not_in_list;
	end;


procedure slList.free;
	begin
		DisposHandle(Handle(itsItems));
		DisposHandle(Handle(itsIndices));
		DisposHandle(Handle(self));
	end;


procedure slList.append (g: sl);
	var
		curSize: longint;
		n, longp, i: longint;
		lp: longPtr;
	begin
		n := numOfItems;
		curSize := n * itemRecSize;
		if GetHandleSize(Handle(itsItems)) < (curSize + itemRecSize) then
			SetHandleSize(Handle(itsItems), curSize + lBlkSize * itemRecSize);
		matrixP(itsItems, n)^ := g;
		setMark(n, not_defined);

		longp := prime;
		with g do
			if (a = 0) or (d = 0) then
				i := ((a * longp) + d) * longp + c
			else
				i := (((longp + a) * longp) + b) * longp + c;
		indexP(itsIndices, i)^ := n;

		numOfItems := n + 1
	end;


function slList.getMat (i: longint): sl;
	begin
		getMat := matrixP(itsItems, i)^
	end;


function slList.getMark (i: longint): integer;
	begin
		getMark := markP(itsItems, i)^
	end;


procedure slList.setMark (i: longint; m: integer);
	begin
		markP(itsItems, i)^ := m
	end;


function slList.index (g: gl): longint;
	var
		longp, i: longint;
	begin
		longp := prime;
		with g do
			if (a = 0) or (d = 0) then
				i := ((a * longp) + d) * longp + c
			else
				i := (((longp + a) * longp) + b) * longp + c;
		index := indexP(itsIndices, i)^
	end;



procedure slList.listSLs;
	var
		i, j, k, l, a, b, c, d, bc: fp;

	function sigma (n: integer): integer;
		begin
			if odd(n) then
				sigma := (n + 1) div 2
			else
				sigma := prime - (n div 2)
		end;

	procedure register (a, b, c, d: integer);
		var
			g, h: sl;
		begin
			glSet(a, b, c, d, g);
			if index(g) = not_in_list then
				append(g);
			h := slInv(g);
			if index(h) = not_in_list then
				append(h);
			with g do
				glSet(-a, -b, -c, -d, h);
			if index(h) = not_in_list then
				append(h);
			h := slInv(h);
			if index(h) = not_in_list then
				append(h)
		end;

	begin
		for i := 1 to prime - 1 do
			begin
				a := sigma(i);
				d := fpInv(a);
				register(a, 0, 0, d)
			end;
		for j := 1 to prime - 1 do
			register(1, sigma(j), 0, 1);
		for k := 1 to prime - 1 do
			register(1, 0, sigma(k), 1);
		for i := 0 to prime - 1 do
			for l := 0 to prime - 1 do
				begin
					a := sigma(i);
					d := sigma(l);
					bc := fpDif(fpProd(a, d), 1);
					if bc = 0 then
						begin
							for j := 0 to prime - 1 do
								register(a, sigma(j), 0, d);
							for k := 0 to prime - 1 do
								register(a, 0, sigma(k), d);
						end
					else
						for j := 1 to prime - 1 do
							begin
								b := sigma(j);
								c := fpQuot(bc, b);
								register(a, b, c, d)
							end
				end
	end;


procedure slList.show;
	var
		n, i: longint;
	begin
		n := numOfItems;
		writeln('numOfItems =', n : 4);
		for i := 0 to n - 1 do
			writeln(gl2str(getMat(i)), getMark(i) : 4)
	end;


end.