unit small_gl2p;   { 1993.6.12  programmed by Masaaki Wada }

interface

uses
	str;

const
	undefined = -1;

type
	fp = -1..127;
	gl = packed record
			a, b, c, d: fp
		end;
	sl = gl;

var
	prime: integer;

procedure fpInit (p: integer);   { 1 < p <= 127 }
function fpNormalize (a: integer): fp;
function fpNeg (a: fp): fp;
function fpSum (a, b: fp): fp;
function fpDif (a, b: fp): fp;
function fpProd (a, b: fp): fp;
function fpInv (a: fp): fp;
function fpQuot (a, b: fp): fp;
function fp2str (a: fp): string;

procedure glSet (a, b, c, d: integer; var g: gl);
function glEQ (g, h: gl): boolean;
function glDet (g: gl): fp;
function glProd (g, h: gl): gl;
function glInv (g: gl): gl;
function slInv (g: sl): sl;
function slConjL (g, h: sl): sl;   { g.h.g^-1 }
function slConjR (g, h: sl): sl;   { g^-1.h.g }
function gl2str (g: gl): string;




implementation


var
	prime2: integer; { prime2 = prime div 2 }
	fpLength: integer;
	fpInvArray: array[fp] of fp;


function fpInv1 (a: fp): fp;
forward;

procedure fpInit (p: integer);   { 1 < p <= 127 }
	var
		a: fp;
	begin
		if (p <= 1) or (p > 127) then
			prime := 2;
		prime := p;
		prime2 := p div 2;
		fpInvArray[-1] := -1;
		fpInvArray[0] := -1;
		for a := 1 to prime - 1 do
			fpInvArray[a] := fpInv1(a);
		if prime <= 2 then
			fpLength := 1
		else
			fpLength := length(long2str(-prime2))
	end;


function fpNormalize (a: integer): fp;
	begin
		a := a mod prime;
		if a < 0 then
			a := a + prime;
		fpNormalize := a
	end;


function fpNeg (a: fp): fp;
	begin
		if a <= 0 then
			fpNeg := a
		else
			fpNeg := prime - a
	end;


function fpSum (a, b: fp): fp;
	begin
		if (a >= 0) and (b >= 0) then
			fpSum := (a + b) mod prime
		else
			fpSum := undefined
	end;


function fpDif (a, b: fp): fp;
	begin
		if (a >= 0) and (b >= 0) then
			fpDif := (prime + a - b) mod prime
		else
			fpDif := undefined
	end;


function fpProd (a, b: fp): fp;
	begin
		if (a >= 0) and (b >= 0) then
			fpProd := (a * b) mod prime
		else
			fpProd := undefined
	end;


function fpInv1 (a: fp): fp;
	var
		x1, y1, x2, y2, q, r: integer;
    { a * xi = yi mod prime }
	begin
		if a > 0 then
			begin
				x1 := 0;
				y1 := prime;
				x2 := 1;
				y2 := a;
				while y2 <> 0 do
					begin
						q := y1 div y2;
						r := y1 - y2 * q;
						y1 := y2;
						y2 := r;
						r := fpDif(x1, fpProd(x2, q));
						x1 := x2;
						x2 := r
					end;
				if y1 = 1 then
					fpInv1 := x1
				else
					fpInv1 := undefined
			end
		else
			fpInv1 := undefined
	end;


function fpInv (a: fp): fp;
	begin
		fpInv := fpInvArray[a]
	end;


function fpQuot (a, b: fp): fp;
	begin
		fpQuot := fpProd(a, fpInv(b))
	end;


function fp2str (a: fp): string;
	var
		n: integer;
		s: string;
	begin
		if (a >= 0) and (a < prime) then
			begin
				if a <= prime2 then
					n := a
				else
					n := a - prime;
				s := long2str(n)
			end
		else
			s := '?';
		fp2str := strStuffL(s, ' ', fpLength)
	end;




procedure glSet (a, b, c, d: integer; var g: gl);
	begin
		g.a := fpNormalize(a);
		g.b := fpNormalize(b);
		g.c := fpNormalize(c);
		g.d := fpNormalize(d)
	end;


function glEQ (g, h: gl): boolean;
	begin
		glEQ := false;
		if g.a = h.a then
			if g.b = h.b then
				if g.c = h.c then
					if g.d = h.d then
						glEQ := true
	end;


function glDet (g: gl): fp;
	begin
		with g do
			glDet := fpNormalize(a * d - b * c)
	end;


function glProd (g, h: gl): gl;
	var
		f: gl;
	begin
		f.a := (g.a * h.a + g.b * h.c) mod prime;
		f.b := (g.a * h.b + g.b * h.d) mod prime;
		f.c := (g.c * h.a + g.d * h.c) mod prime;
		f.d := (g.c * h.b + g.d * h.d) mod prime;
		glProd := f
	end;


function glInv (g: gl): gl;
	var
		r: fp;
		h: gl;
	begin
		r := fpInv(glDet(g));
		h.a := fpProd(r, g.d);
		h.b := fpProd(r, fpNeg(g.b));
		h.c := fpProd(r, fpNeg(g.c));
		h.d := fpProd(r, g.a);
		glInv := h
	end;


function slInv (g: sl): sl;
	var
		h: gl;
	begin
		h.a := g.d;
		h.b := fpNeg(g.b);
		h.c := fpNeg(g.c);
		h.d := g.a;
		slInv := h
	end;


function slConjL (g, h: sl): sl;
	begin
		slConjL := glProd(glProd(g, h), slInv(g))
	end;


function slConjR (g, h: sl): sl;
	begin
		slConjR := glProd(slInv(g), glProd(h, g))
	end;


function gl2str (g: gl): string;
	begin
		with g do
			gl2str := concat('(', fp2str(a), ',', fp2str(b), ';', fp2str(c), ',', fp2str(d), ')')
	end;


end.