unit complex;   { programmed by Masaaki Wada 1991.4.28 }

interface

type
	complex = record
			re, im: extended
		end;

procedure setComplex (var z: complex; x, y: extended);

function complexAbs (z: complex): extended;
function complexArg (z: complex): extended;

procedure complexScale (r: extended; var w: complex);
procedure complexNeg (var z: complex);
procedure complexConj (var z: complex);
procedure complexInv (var z: complex);
procedure complexAdd (z: complex; var w: complex);
procedure complexSub (z: complex; var w: complex);
procedure complexMul (z: complex; var w: complex);
procedure complexDiv (z: complex; var w: complex);
procedure complexExp (z: complex; var w: complex);
procedure complexLog (z: complex; var w: complex);



implementation


procedure setComplex (var z: complex; x, y: extended);
	begin
		z.re := x;
		z.im := y
	end;


function complexAbs (z: complex): extended;
	begin
		complexAbs := sqrt(sqr(z.re) + sqr(z.im))
	end;


function complexArg (z: complex): extended;
	var
		x, y, t: extended;
		n: integer;
	begin
		if z.re < 0 then
			if z.im >= 0 then
				begin
					x := z.im;
					y := -z.re;
					n := 1
				end
			else
				begin
					x := -z.im;
					y := z.re;
					n := -1
				end
		else
			begin
				x := z.re;
				y := z.im;
				n := 0
			end;
		if y > x then
			begin
				t := x;
				x := y;
				y := -t;
				n := n + 1
			end
		else if y < -x then
			begin
				t := x;
				x := -y;
				y := t;
				n := n - 1
			end;
		if x = 0 then
			complexArg := 0.0
		else
			complexArg := arctan(y / x) + n * pi / 2
	end;


procedure complexScale (r: extended; var w: complex);
	begin
		w.re := r * w.re;
		w.im := r * w.im
	end;


procedure complexNeg (var z: complex);
	begin
		z.re := -z.re;
		z.im := -z.im
	end;


procedure complexConj (var z: complex);
	begin
		z.im := -z.im
	end;


procedure complexInv (var z: complex);
	var
		a: extended;
	begin
		a := 1.0 / sqr(complexAbs(z));
		complexConj(z);
		complexScale(a, z)
	end;


procedure complexAdd (z: complex; var w: complex);
	begin
		w.re := w.re + z.re;
		w.im := w.im + z.im
	end;


procedure complexSub (z: complex; var w: complex);
	begin
		w.re := w.re - z.re;
		w.im := w.im - z.im
	end;


procedure complexMul (z: complex; var w: complex);
	var
		z1: complex;
	begin
		z1 := w;
		w.re := z.re * z1.re - z.im * z1.im;
		w.im := z.re * z1.im + z1.re * z.im
	end;


procedure complexDiv (z: complex; var w: complex);
	begin
		complexInv(z);
		complexMul(z, w)
	end;


procedure complexExp (z: complex; var w: complex);
	var
		r: extended;
	begin
		r := exp(z.re);
		w.re := r * cos(z.im);
		w.im := r * sin(z.im)
	end;


procedure complexLog (z: complex; var w: complex);
	begin
		w.re := ln(complexAbs(z));
		w.im := complexArg(z)
	end;


end.