unit ecm;   { August 9, 1991  programmed by Masaaki Wada }

interface

	uses
		nat, prime;

	function newEcm (nm, nl: nat): Handle;
	procedure disposeEcm (h: Handle);

	procedure addEcm (h: Handle; nbx, ncx, nby, ncy, nbxmy, ncxmy, nbxpy, ncxpy: nat);
	procedure doubleEcm (h: Handle; nbx, ncx, nb2x, nc2x: nat);
	procedure powerEcm (h: Handle; nb, nc, ne: nat);
	procedure secondEcm (h: Handle; nb, nc, nbx: nat; ptbl: Handle; limit: longint);


implementation


	const
		numTempNats = 140;

	type
		ecmRecord = record
				nm, nl: nat;
				nt: array[1..numTempNats] of nat;
			end;
		ecmPtr = ^ecmRecord;
		ecmHandle = ^ecmPtr;


	function newEcm (nm, nl: nat): Handle;
		var
			h: Handle;
			n: nat;
			sz: longint;
			i: integer;
	begin
		h := NewHandle(SizeOf(ecmRecord));
		nNew(n);
		nCopy(nm, n);
		ecmHandle(h)^^.nm := n;
		nNew(n);
		nCopy(nl, n);
		ecmHandle(h)^^.nl := n;
		sz := nm^^.len * 2 + 1;
		for i := 1 to numTempNats do
			begin
				nNewAtLeast(n, sz);
				ecmHandle(h)^^.nt[i] := n
			end;
		newEcm := h
	end;

	procedure disposeEcm (h: Handle);
		var
			i: integer;
	begin
		nDispose(ecmHandle(h)^^.nm);
		nDispose(ecmHandle(h)^^.nl);
		for i := 1 to numTempNats do
			nDispose(ecmHandle(h)^^.nt[i]);
		DisposHandle(h)
	end;




	procedure nAddM (n1, n2, nm, nd: nat);
		var
			p1, p2, pm, pd: Ptr;
	begin
		p1 := Ptr(longint(n1^) + 4);
		p2 := Ptr(longint(n2^) + 4);
		pm := Ptr(longint(nm^) + 4);
		pd := Ptr(longint(nd^) + 4);
		aAdd(p1, n1^^.len, p2, n2^^.len, pd, nd^^.len);
		if aCompare(pd, nd^^.len, pm, nm^^.len) in [gt, eq] then
			aSub(pd, nd^^.len, pm, nm^^.len, pd, nd^^.len)
	end;


	procedure nSubM (n1, n2, nm, nd: nat);
		var
			p1, p2, pm, pd: Ptr;
	begin
		p1 := Ptr(longint(n1^) + 4);
		p2 := Ptr(longint(n2^) + 4);
		pm := Ptr(longint(nm^) + 4);
		pd := Ptr(longint(nd^) + 4);
		if aCompare(p1, n1^^.len, p2, n2^^.len) in [gt, eq] then
			aSub(p1, n1^^.len, p2, n2^^.len, pd, nd^^.len)
		else
			begin
				aAdd(p1, n1^^.len, pm, nm^^.len, pd, nd^^.len);
				aSub(pd, nd^^.len, p2, n2^^.len, pd, nd^^.len)
			end
	end;


	procedure nMulM (n1, n2, nm, nd: nat);
		var
			p1, p2, pm, pd: Ptr;
			ldummy: longint;
	begin
		p1 := Ptr(longint(n1^) + 4);
		p2 := Ptr(longint(n2^) + 4);
		pm := Ptr(longint(nm^) + 4);
		pd := Ptr(longint(nd^) + 4);
		aMul(p1, n1^^.len, p2, n2^^.len, pd, nd^^.len);
		aDiv(pd, nd^^.len, pm, nm^^.len, nil, ldummy)
	end;


	procedure nCopyQ (ns, nd: nat);
		var
			l: longint;
	begin
		l := ns^^.len;
		if l < 0 then
			l := 0;
		BlockMove(Ptr(ns^), Ptr(nd^), l * 4 + 4)
	end;


	procedure addEcm (h: Handle; nbx, ncx, nby, ncy, nbxmy, ncxmy, nbxpy, ncxpy: nat);
	begin
		with ecmHandle(h)^^ do
			begin
				nAddM(nbx, ncx, nm, nt[1]);     { t1 = b_x + c_x }
				nSubM(nbx, ncx, nm, nt[2]);     { t2 = b_x - c_x }
				nAddM(nby, ncy, nm, nt[3]);     { t3 = b_y + c_y }
				nSubM(nby, ncy, nm, nt[4]);     { t4 = b_y - c_y }
				nCopyQ(nbxmy, nt[9]);           { in case nbxmy = nbxpy }

				nMulM(nt[1], nt[4], nm, nt[5]);
				nMulM(nt[2], nt[3], nm, nt[6]);

				nSubM(nt[5], nt[6], nm, nt[7]);
				nMulM(nt[7], nt[7], nm, nt[8]);
				nMulM(nt[8], ncxmy, nm, nbxpy); { b_(x+y) = c_(x-y)(t1 t4 - t2 t3)^2 }

				nAddM(nt[5], nt[6], nm, nt[7]);
				nMulM(nt[7], nt[7], nm, nt[8]);
				nMulM(nt[8], nt[9], nm, ncxpy); { c_(x+y) = b_(x-y)(t1 t4 + t2 t3)^2 }
			end
	end;



	procedure doubleEcm (h: Handle; nbx, ncx, nb2x, nc2x: nat);
	begin
		with ecmHandle(h)^^ do
			begin
				nAddM(nbx, ncx, nm, nt[1]);     { t1 = b_x + c_x }
				nSubM(nbx, ncx, nm, nt[2]);     { t2 = b_x - c_x }

				nMulM(nt[1], nt[1], nm, nt[3]); { t3 = t1^2 }
				nMulM(nt[2], nt[2], nm, nt[4]); { t4 = t2^2 }
				nMulM(nt[3], nt[4], nm, nc2x);  { c_2x = t1^2 t2^2 }

				nSubM(nt[3], nt[4], nm, nt[5]); { t5 = t1^2 - t2^2 }
				nMulM(nt[5], nl, nm, nt[6]);
				nAddM(nt[6], nt[4], nm, nt[7]);
				nMulM(nt[5], nt[7], nm, nb2x);  { b_2x = (t1^2 - t2^2)(t2^2 + L(t1^2 - t2^2)) }
			end
	end;




	procedure powerEcm (h: Handle; nb, nc, ne: nat);
		var
			nb0, nc0, nb1, nc1: nat;
			pe: Ptr;
			k, i: longint;
	begin
		if nDefined(nb) and nDefined(nc) and (nLen(ne) > 0) then
			begin
				with ecmHandle(h)^^ do
					begin
						nb0 := nt[10];
						nc0 := nt[11];
						nb1 := nt[12];
						nc1 := nt[13]
					end;
				nb0^^.len := 0;
				nc0^^.len := 1;
				nc0^^.val[0] := 1;
				nCopyQ(nb, nb1);
				nCopyQ(nc, nc1);
				pe := Ptr(longint(ne^) + 4);
				k := ne^^.len * 32 - 1;
				while not aBitTst(pe, k) do
					k := k - 1;
				for i := k downto 1 do
					if aBitTst(pe, i) then
						begin
							addEcm(h, nb1, nc1, nb0, nc0, nb, nc, nb0, nc0);
							doubleEcm(h, nb1, nc1, nb1, nc1)
						end
					else
						begin
							addEcm(h, nb1, nc1, nb0, nc0, nb, nc, nb1, nc1);
							doubleEcm(h, nb0, nc0, nb0, nc0)
						end;
				if aBitTst(pe, 0) then
					addEcm(h, nb1, nc1, nb0, nc0, nb, nc, nb0, nc0)
				else
					doubleEcm(h, nb0, nc0, nb0, nc0);
				nCopy(nb0, nb);
				nCopy(nc0, nc)
			end
	end;



	procedure secondEcm (h: Handle; nb, nc, nbx: nat; ptbl: Handle; limit: longint);
		var
			nm: nat;
			ntb0, ntc0, ntb1, ntc1: array[0..31] of nat;
			prevp, curp: longint;
			assignNum, k, i: integer;
		procedure assignNat (var n: nat);
		begin
			assignNum := assignNum + 1;
			n := ecmHandle(h)^^.nt[assignNum]
		end;
	begin
		if nUndefined(nb) or nUndefined(nc) then
			nUndefine(nbx)
		else
			begin
				nm := ecmHandle(h)^^.nm;
				assignNum := 9;
				for i := 0 to 31 do
					begin
						assignNat(ntb0[i]);
						assignNat(ntc0[i]);
						assignNat(ntb1[i]);
						assignNat(ntc1[i])
					end;
				curp := 0;
				for i := 0 to 31 do
					begin
						ntb0[i]^^.len := 0;
						ntc0[i]^^.len := 1;
						ntc0[i]^^.val[0] := 1;
						nCopyQ(nb, ntb1[i]);
						nCopyQ(nc, ntc1[i])
					end;
				long2nat(1, nbx);
				while peekNextPrime(ptbl) < limit do
					begin
						prevp := curp;
						curp := nextPrime(ptbl);
						k := 31;
						while BTST(prevp, k) = BTST(curp, k) do
							k := k - 1;
						for i := k downto 1 do
							if BTST(curp, i) then
								begin
									addEcm(h, ntb1[i + 1], ntc1[i + 1], ntb0[i + 1], ntc0[i + 1], nb, nc, ntb0[i], ntc0[i]);
									doubleEcm(h, ntb1[i + 1], ntc1[i + 1], ntb1[i], ntc1[i])
								end
							else
								begin
									addEcm(h, ntb1[i + 1], ntc1[i + 1], ntb0[i + 1], ntc0[i + 1], nb, nc, ntb1[i], ntc1[i]);
									doubleEcm(h, ntb0[i + 1], ntc0[i + 1], ntb0[i], ntc0[i])
								end;
						if BTST(curp, 0) then
							addEcm(h, ntb1[1], ntc1[1], ntb0[1], ntc0[1], nb, nc, ntb0[0], ntc0[0])
						else
							doubleEcm(h, ntb0[1], ntc0[1], ntb0[0], ntc0[0]);
						nMulMod(nbx, ntb0[0], nm, nbx)
					end;
			end
	end;





end.