unit fp;   { February 16, 1991  programmed by Masaaki Wada }



interface


uses
	str;

const
	infinite = -1;
	undefined = -2;

var
	prime, prime2: integer;

procedure fpInit (p: integer);

function fpNormalize (a: integer): integer;     { a mod p }
function fpNeg (a: integer): integer;               { - a }
function fpSum (a, b: integer): integer;          { a + b }
function fpDif (a, b: integer): integer;          { a - b }
function fpProd (a, b: integer): integer;         { a * b }
function fpInv (a: integer): integer;              { a^-1 }
function fpQuot (a, b: integer): integer;         { a / b }
function fpPower (a: integer; e: longint): integer; { a^e }

procedure fpLongStr (long: boolean);
function fp2str (a: integer): string;




implementation

var
	fpLong: boolean;
	fpLength: integer;


procedure fpInit (p: integer);
	begin
		prime := p;
		prime2 := p div 2;
		fpLong := false;
		if prime <= 2 then
			fpLength := 1
		else
			fpLength := length(long2str(-prime2))
	end;



function fpNormalize (a: integer): integer;
	begin
		a := a mod prime;
		if a < 0 then
			a := a + prime;
		fpNormalize := a
	end;


function fpNeg (a: integer): integer;
	begin
		if a <= 0 then
			fpNeg := a
		else
			fpNeg := prime - a
	end;


function fpSum (a, b: integer): integer;
	begin
		if (a >= 0) and (b >= 0) then
			begin
				a := a + b;
				if a >= prime then
					a := a - prime;
				fpSum := a
			end
		else if (a = infinite) and (b = infinite) then
			fpSum := infinite
		else
			fpSum := undefined
	end;


function fpDif (a, b: integer): integer;
	begin
		if (a >= 0) and (b >= 0) then
			begin
				a := a - b;
				if a < 0 then
					a := a + prime;
				fpDif := a
			end
		else if (a = infinite) and (b = infinite) then
			fpDif := infinite
		else
			fpDif := undefined
	end;


function fpProdX (a, b, p: integer): integer;
external;

function fpProd (a, b: integer): integer;
	begin
		if a > 0 then
			if b > 0 then
				fpProd := fpProdX(a, b, prime)
			else
				fpProd := b
		else if a = 0 then
			if b >= 0 then
				fpProd := 0
			else
				fpProd := undefined
		else if a = infinite then
			if (b > 0) or (b = infinite) then
				fpProd := infinite
			else
				fpProd := undefined
		else
			fpProd := undefined
	end;


function fpInv (a: integer): integer;
	var
		x1, y1, x2, y2, q, r: integer;
    { a * xi = yi mod prime }
	begin
		if a > 0 then
			begin
				x1 := 0;
				y1 := prime;
				x2 := 1;
				y2 := a;
				while y2 <> 0 do
					begin
						q := y1 div y2;
						r := y1 - y2 * q;
						y1 := y2;
						y2 := r;
						r := fpDif(x1, fpProd(x2, q));
						x1 := x2;
						x2 := r
					end;
				if y1 = 1 then
					fpInv := x1
				else
					fpInv := undefined
			end
		else if a = 0 then
			fpInv := infinite
		else if a = infinite then
			fpInv := 0
		else
			fpInv := undefined
	end;


function fpQuot (a, b: integer): integer;
	begin
		fpQuot := fpProd(a, fpInv(b))
	end;


function fpPower (a: integer; e: longint): integer;
	var
		b: integer;
	begin
		if e = 0 then
			fpPower := 1
		else
			begin
				if e < 0 then
					begin
						a := fpInv(a);
						e := -e
					end;
				if a <= 1 then
					fpPower := a
				else
					begin
						if odd(e) then
							b := a
						else
							b := 1;
						e := BSR(e, 1);
						while e <> 0 do
							begin
								a := fpProd(a, a);
								if odd(e) then
									b := fpProd(a, b);
								e := BSR(e, 1)
							end;
						fpPower := b
					end
			end
	end;



procedure fpLongStr (long: boolean);
	begin
		fpLong := long
	end;



function fp2str (a: integer): string;
	var
		s: string;
	begin
		if (a >= 0) and (a < prime) then
			begin
				if a > prime2 then
					a := a - prime;
				s := long2str(a)
			end
		else if a = infinite then
			s := ''
		else
			s := '?';
		if fpLong then
			fp2str := strStuffL(s, ' ', fpLength)
		else
			fp2str := s
	end;



end.