unit nat;   { August 6, 1991  programmed by Masaaki Wada }


interface


uses
	SANE, str;


const
	maxSize = 8000;


type
	long = longint;   { unsigned longint }

	natRecord = record
			len: longint;
			val: array[0..maxSize] of long
		end;
	natPtr = ^natRecord;
	nat = ^natPtr;

	compareResult = (lt, eq, gt, nc);


function aBitTst (p: univ Ptr; l: longint): boolean;

procedure aLongAdd (w: long; ps, pd: univ Ptr; var l: longint);
procedure aLongSub (w: long; ps, pd: univ Ptr; var l: longint);
procedure aLongMul (w: long; ps, pd: univ Ptr; var l: longint);
procedure aLongDiv (w: long; ps, pd: univ Ptr; var len: longint; var r: long);
function aLongMod (w: long; p: univ Ptr; len: longint): long;

function aCompare (p1: univ Ptr; l1: longint; p2: univ Ptr; l2: longint): compareResult;
procedure aAdd (p1: univ Ptr; l1: longint; p2: univ Ptr; l2: longint; pd: univ Ptr; var ld: longint);
procedure aSub (p1: univ Ptr; l1: longint; p2: univ Ptr; l2: longint; pd: univ Ptr; var ld: longint);
procedure aMul (p1: univ Ptr; l1: longint; p2: univ Ptr; l2: longint; pd: univ Ptr; var ld: longint);
procedure aDiv (r: univ Ptr; var lr: longint; p: univ Ptr; lp: longint; q: univ Ptr; var lq: longint);


procedure nNew (var n: nat);
procedure nNewAtLeast (var n: nat; l: longint);
procedure nDispose (var n: nat);
function nSize (n: nat): longint;
procedure nSizeAtLeast (n: nat; siz: longint);

procedure nUndefine (n: nat);
function nDefined (n: nat): boolean;
function nUndefined (n: nat): boolean;
procedure nSetLen (n: nat; l: longint);
function nLen (n: nat): longint;
procedure nSetVal (n: nat; i: longint; v: long);
function nVal (n: nat; i: longint): long;
procedure nNormalize (n: nat);

procedure long2nat (w: long; n: nat);
procedure comp2nat (c: comp; n: nat);
function nat2x (n: nat): extended;

procedure nCopy (ns, nd: nat);

function nLongCompare (n: nat; w: long): compareResult;
procedure nLongAdd (ns: nat; w: long; nd: nat);
procedure nLongSub (ns: nat; w: long; nd: nat);
procedure nLongMul (ns: nat; w: long; nd: nat);
procedure nLongDiv (ns: nat; w: long; nq: nat; var r: long);
function nLongMod (n: nat; w: long): long;

function nCompare (n1, n2: nat): compareResult;
procedure nAdd (n1, n2, nd: nat);   { n1 + n2 ---> nd }
procedure nSub (n1, n2, nd: nat);   { n1 - n2 ---> nd }
procedure nMul (n1, n2, nd: nat);   { n1 * n2 ---> nd }
procedure nDiv (n1, n2, nq, nr: nat);   { n1 = n2 * nq + nr }
procedure nPower (ns: nat; e: longint; nd: nat);
procedure nSqrt (ns, nd, nr: nat);   { ns = nd^2 + nr }

procedure nMod (n, nm: nat);               { n mod nm ---> n }
procedure nAddMod (n1, n2, nm, nd: nat);   { (n1 + n2) mod nm ---> nd }
procedure nSubMod (n1, n2, nm, nd: nat);   { (n1 - n2) mod nm ---> nd }
procedure nMulMod (n1, n2, nm, nd: nat);   { (n1 * n2) mod nm ---> nd }
procedure nPowerMod (ns, ne, nm, nd: nat); { (ns^ne) mod nm ---> nd }
procedure nInvMod (ns, nm, nd: nat);
procedure nGcd (n1, n2, nd: nat);


procedure str2nat (s: string; n: nat);
function nat2str (n: nat): string;

procedure readnat (n: nat);
procedure writenat (n: nat);




implementation


const
	minSize = 2;
	natIndefinite = -1;


type
	long2 = record   { for long-comp conversion }
			hi, lo: long
		end;



function long2comp (wh, wl: long): comp;
	var
		temp: long2;
	begin
		temp.hi := wh;
		temp.lo := wl;
		long2comp := comp(temp)
	end;


{ Assembly language routines }
procedure lAdd (ah, al, bh, bl: long; var dh, dl: long);
external;   { ah:al + bh:bl ---> dh:dl }
procedure lSub (ah, al, bh, bl: long; var dh, dl: long);
external;   { ah:al - bh:bl ---> dh:dl }
procedure lMul (a, b: long; var dh, dl: long);
external;   { a * b ---> dh:dl }
procedure lDiv (ah, al, b: long; var dq, dr: long);
external;   { ah:al = b * dq + dr,  0 <= dr < b }




procedure nNew (var n: nat);
	begin
		n := nat(NewHandle(4 * minSize + 4));
		nUndefine(n)
	end;



procedure nNewAtLeast (var n: nat; l: longint);
	begin
		if l < minSize then
			l := minSize;
		n := nat(NewHandle(4 * l + 4));
		nUndefine(n)
	end;



procedure nDispose (var n: nat);
	begin
		DisposHandle(Handle(n));
		n := nil
	end;



function nSize (n: nat): longint;
	begin
		nSize := BSR(GetHandleSize(Handle(n)), 2) - 1
	end;



procedure nSizeAtLeast (n: nat; siz: longint);
	begin
		if siz > nSize(n) then
			SetHandleSize(Handle(n), 4 * siz + 4)
	end;



procedure nUndefine (n: nat);
	begin
		n^^.len := natIndefinite
	end;



function nDefined (n: nat): boolean;
	begin
		if n^^.len > nSize(n) then
			nUndefine(n);
		nDefined := (n^^.len >= 0)
	end;



function nUndefined (n: nat): boolean;
	begin
		if n^^.len > nSize(n) then
			nUndefine(n);
		nUndefined := (n^^.len < 0)
	end;



procedure nSetLen (n: nat; l: longint);
	begin
		nSizeAtLeast(n, l);
		n^^.len := l
	end;



function nLen (n: nat): longint;
	begin
		if n^^.len > nSize(n) then
			nUndefine(n);
		nLen := n^^.len
	end;



procedure nSetVal (n: nat; i: longint; v: long);
	begin
		nSizeAtLeast(n, i + 1);
		n^^.val[i] := v
	end;



function nVal (n: nat; i: longint): long;
	begin
		if i < nSize(n) then
			nVal := n^^.val[i]
		else
			nVal := 0
	end;



procedure nNormalize (n: nat);
	begin
		if nLen(n) < 0 then
			nUndefine(n)
		else
			with n^^ do
				while len > 0 do
					if val[len - 1] = 0 then
						len := len - 1
					else
						leave
	end;



procedure long2nat (w: long; n: nat);
	begin
		if w = 0 then
			n^^.len := 0
		else
			begin
				nSetLen(n, 1);
				n^^.val[0] := w
			end
	end;



procedure comp2nat (c: comp; n: nat);
	begin
		if c < 0 then
			nUndefine(n)
		else
			begin
				nSetLen(n, 2);
				n^^.val[1] := long2(c).hi;
				n^^.val[0] := long2(c).lo;
				nNormalize(n)
			end
	end;



function nat2x (n: nat): extended;
	var
		x: extended;
		l, k: longint;
	begin
		if nUndefined(n) then
			nat2x := NAN(20)
		else
			begin
				x := 0;
				l := n^^.len;
				k := l - 3;
				if k < 0 then
					k := 0;
				while l > k do
					begin
						l := l - 1;
						x := Scalb(32, x) + long2comp(0, n^^.val[l])
					end;
				nat2x := Scalb(32 * l, x)
			end
	end;



procedure nCopy (ns, nd: nat);
	var
		l: longint;
	begin
		nNormalize(ns);
		if nd <> ns then
			begin
				l := ns^^.len;
				if l < 0 then
					l := 0;
				nSizeAtLeast(nd, l);
				BlockMove(Ptr(ns^), Ptr(nd^), l * 4 + 4)
			end
	end;



function nLongCompare (n: nat; w: long): compareResult;
	var
		l: longint;
		cv, cw: comp;
	begin
		nNormalize(n);
		l := n^^.len;
		if l < 0 then
			nLongCompare := nc
		else if l > 1 then
			nLongCompare := gt
		else if l = 0 then
			if w = 0 then
				nLongCompare := eq
			else
				nLongCompare := lt
		else
			begin
				cv := long2comp(0, n^^.val[0]);
				cw := long2comp(0, w);
				if cv > cw then
					nLongCompare := gt
				else if cv < cw then
					nLongCompare := lt
				else
					nLongCompare := eq
			end
	end;



procedure nLongAdd (ns: nat; w: long; nd: nat);
	var
		l: longint;
	begin
		l := nLen(ns);
		if l < 0 then
			nUndefine(nd)
		else if l = 0 then
			long2nat(w, nd)
		else
			begin
				nSizeAtLeast(nd, l + 1);
				aLongAdd(w, @ns^^.val[0], @nd^^.val[0], l);
				nd^^.len := l
			end
	end;



procedure nLongSub (ns: nat; w: long; nd: nat);
	var
		l: longint;
	begin
		l := nLen(ns);
		if l < 0 then
			nUndefine(nd)
		else
			begin
				nSizeAtLeast(nd, l);
				aLongSub(w, @ns^^.val[0], @nd^^.val[0], l);
				nd^^.len := l
			end
	end;



procedure nLongMul (ns: nat; w: long; nd: nat);
	var
		l: longint;
	begin
		l := nLen(ns);
		if l < 0 then
			nUndefine(nd)
		else if (l = 0) or (w = 0) then
			long2nat(0, nd)
		else if w = 1 then
			nCopy(ns, nd)
		else
			begin
				nSizeAtLeast(nd, l + 1);
				aLongMul(w, @ns^^.val[0], @nd^^.val[0], l);
				nd^^.len := l
			end
	end;



procedure nLongDiv (ns: nat; w: long; nq: nat; var r: long);
	var
		l: longint;
	begin
		if nUndefined(ns) or (w = 0) then
			nUndefine(nq)
		else if w = 1 then
			begin
				nCopy(ns, nq);
				r := 0
			end
		else
			begin
				l := nLen(ns);
				nSizeAtLeast(nq, l);
				aLongDiv(w, @ns^^.val[0], @nq^^.val[0], l, r);
				nq^^.len := l
			end
	end;



function nLongMod (n: nat; w: long): long;
	begin
		if nUndefined(n) or (w = 0) then
			nLongMod := $FFFFFFFF
		else if (n^^.len = 0) or (w = 1) then
			nLongMod := 0
		else
			nLongMod := aLongMod(w, @n^^.val[0], n^^.len)
	end;



function nCompare (n1, n2: nat): compareResult;
	begin
		nNormalize(n1);
		nNormalize(n2);
		nCompare := aCompare(@n1^^.val[0], n1^^.len, @n2^^.val[0], n2^^.len)
	end;



procedure nAdd (n1, n2, nd: nat);   { n1 + n2 ---> nd }
	var
		l1, l2: longint;
	begin
		if nUndefined(n1) or nUndefined(n2) then
			nUndefine(nd)
		else
			begin
				l1 := n1^^.len;
				l2 := n2^^.len;
				if l1 >= l2 then
					nSizeAtLeast(nd, l1 + 1)
				else
					nSizeAtLeast(nd, l2 + 1);
				aAdd(@n1^^.val[0], l1, @n2^^.val[0], l2, @nd^^.val[0], nd^^.len)
			end
	end;



procedure nSub (n1, n2, nd: nat);   { n1 - n2 ---> nd }
	var
		l1, l2: longint;
	begin
		case nCompare(n1, n2) of
			lt, nc: 
				nUndefine(nd);
			eq: 
				long2nat(0, nd);
			gt: 
				begin
					l1 := n1^^.len;
					l2 := n2^^.len;
					nSetLen(nd, l1);
					aSub(@n1^^.val[0], l1, @n2^^.val[0], l2, @nd^^.val[0], nd^^.len)
				end
		end
	end;



procedure nMul (n1, n2, nd: nat);   { n1 * n2 ---> nd }
	var
		l1, l2: longint;
		temp: nat;
	begin
		l1 := nLen(n1);
		l2 := nLen(n2);
		if (l1 < 0) or (l2 < 0) then
			nUndefine(nd)
		else if (l1 = 0) or (l2 = 0) then
			long2nat(0, nd)
		else
			begin
				nNew(temp);
				nSetLen(temp, l1 + l2);
				aMul(@n1^^.val[0], l1, @n2^^.val[0], l2, @temp^^.val[0], temp^^.len);
				nCopy(temp, nd);
				nDispose(temp)
			end
	end;



procedure nDiv (n1, n2, nq, nr: nat);   { n1 = n2 * nq + nr }
	var
		l1, l2: longint;
		r: long;
		np: nat;
	begin
		nNormalize(n1);
		nNormalize(n2);
		l1 := n1^^.len;
		l2 := n2^^.len;
		if (l1 < 0) or (l2 <= 0) then
			begin
				nUndefine(nq);
				nUndefine(nr)
			end
		else if l2 = 1 then
			begin
				nLongDiv(n1, n2^^.val[0], nq, r);
				long2nat(r, nr)
			end
		else if l1 < l2 then
			begin
				nCopy(n1, nr);
				long2nat(0, nq)
			end
		else
			begin
				nNew(np);
				nCopy(n2, np);
				nSizeAtLeast(nr, l1 + 1);
				nCopy(n1, nr);
				nSizeAtLeast(nq, l1 + 1 - l2);
				aDiv(@nr^^.val[0], nr^^.len, @np^^.val[0], np^^.len, @nq^^.val[0], nq^^.len);
				nDispose(np)
			end
	end;



procedure nPower (ns: nat; e: longint; nd: nat);
	var
		l: longint;
		n: nat;
	begin
		l := nLen(ns);
		if (l < 0) or (e < 0) or ((l = 0) and (e = 0)) then
			nUndefine(nd)
		else
			begin
				nNew(n);
				nCopy(ns, n);
				if odd(e) then
					nCopy(n, nd)
				else
					long2nat(1, nd);
				e := BSR(e, 1);
				while e > 0 do
					begin
						nMul(n, n, n);
						if odd(e) then
							nMul(n, nd, nd);
						e := BSR(e, 1)
					end;
				nDispose(n)
			end
	end;




procedure nSqrt (ns, nd, nr: nat);   { ns = nd^2 + nr }

	type
		word2 = record
				hi, lo: integer
			end;

	var
		l, l2, kr0, krm, kr1, kd0, kd1, i, j: longint;
		x: extended;
		v, q, dummy, p, q2, a, ah, al, c, d: long;
		r: comp;

	function w2l (h, l: integer): long;
		var
			v: long;
		begin
			word2(v).hi := h;
			word2(v).lo := l;
			w2l := v
		end;

	function vw (n: nat; k: longint): integer;
		begin
			if odd(k) then
				vw := HiWord(n^^.val[k div 2])
			else
				vw := LoWord(n^^.val[k div 2])
		end;

	procedure setvw (n: nat; k: longint; w: integer);
		begin
			if odd(k) then
				word2(n^^.val[k div 2]).hi := w
			else
				word2(n^^.val[k div 2]).lo := w
		end;

	function vl (n: nat; k: longint): long;
		begin
			vl := w2l(vw(n, k + 1), vw(n, k))
		end;

	procedure setvl (n: nat; k: longint; v: long);
		begin
			setvw(n, k, LoWord(v));
			setvw(n, k + 1, HiWord(v))
		end;

	procedure nSqrt1 (x: extended; var d: long; var r: comp);
		var
			c: comp;
			rd: RoundDir;
		begin
			rd := GetRound;
			SetRound(Downward);
			c := Rint(sqrt(x));
			SetRound(rd);
			d := long2(c).lo;
			r := x - c * c
		end;

	begin
		l := nLen(ns);
		if l < 0 then
			begin
				nUndefine(nd);
				nUndefine(nr)
			end
		else if l = 0 then
			begin
				long2nat(0, nd);
				long2nat(0, nr)
			end
		else if l <= 2 then
			begin
				nSqrt1(nat2x(ns), v, r);
				long2nat(v, nd);
				comp2nat(r, nr)
			end
		else
			begin
				nCopy(ns, nr);
				l2 := (l + 1) div 2;
				nSizeAtLeast(nd, l2 + 1);
				nd^^.len := l2;
				setvl(nd, l, 0);
				kr1 := 2 * l;
				kd1 := l;

      { first approximation by SANE }
				x := long2comp(0, vl(nr, kr1 - 2));
				x := Scalb(32, x) + long2comp(0, vl(nr, kr1 - 4));
				nSqrt1(x, v, r);
				setvl(nd, kd1 - 2, v);
				setvl(nr, kr1 - 2, long2(r).hi);
				setvl(nr, kr1 - 4, long2(r).lo);

				for i := 3 to kd1 do
					begin
						kd0 := kd1 - i;
						kr0 := 2 * kd0;
						krm := kr0 + i;
						lDiv(vl(nr, krm), vl(nr, krm - 2), v, q, dummy);
						q := q div 2;
						if q > $0000FFFF then
							q := $0000FFFF;
						setvw(nd, kd0, LoWord(q));
						lMul(q, q, dummy, p);
						lSub(0, w2l(0, vw(nr, kr0)), 0, p, dummy, a);
						setvw(nr, kr0, LoWord(a));
						c := w2l(0, -HiWord(a));
						q2 := 2 * q;
						j := 1;
						while j < i do
							begin
								lMul(vl(nd, kd0 + j), q2, ah, al);
								lAdd(ah, al, 0, c, ah, al);
								lSub(0, vl(nr, kr0 + j), ah, al, ah, al);
								setvl(nr, kr0 + j, al);
								c := -ah;
								j := j + 2
							end;
						lSub(0, vl(nr, kr0 + j), 0, c, ah, al);
						setvl(nr, kr0 + j, al);
						if ah <> 0 then   { add back }
							repeat
								q := q - 1;
								setvw(nd, kd0, LoWord(q));
								c := 2 * q + 1;
								lAdd(0, w2l(0, vw(nr, kr0)), 0, c, dummy, a);
								setvw(nr, kr0, LoWord(a));
								c := w2l(0, HiWord(a));
								j := 1;
								while j < i do
									begin
										lMul(vl(nd, kd0 + j), 2, ah, al);
										lAdd(ah, al, 0, c, ah, al);
										lAdd(0, vl(nr, kr0 + j), ah, al, c, d);
										setvl(nr, kr0 + j, d);
										j := j + 2
									end;
								lAdd(0, vl(nr, kr0 + j), 0, c, c, d);
								setvl(nr, kr0 + j, d)
							until c <> 0
					end;
				nr^^.len := l2 + 1;
				nNormalize(nr);
				nNormalize(nd);
			end
	end;



procedure nMod (n, nm: nat);   { n mod nm ---> n }
	var
		l, lm, ldummy: longint;
	begin
		l := nLen(n);
		lm := nLen(nm);
		if (l < 0) or (lm < 0) then
			nUndefine(n)
{   else if lm = 0 then do nothing }
		else if lm > 0 then
			begin
				nSizeAtLeast(n, l + 1);
				aDiv(@n^^.val[0], n^^.len, @nm^^.val[0], lm, nil, ldummy)
			end
	end;



procedure nAddMod (n1, n2, nm, nd: nat);   { (n1 + n2) mod nm ---> nd }
	var
		ndx: nat;
	begin
		nNew(ndx);
		nAdd(n1, n2, ndx);
		nMod(ndx, nm);
		nCopy(ndx, nd);
		nDispose(ndx)
	end;



procedure nSubMod (n1, n2, nm, nd: nat);   { (n1 - n2) mod nm ---> nd }
	var
		n2x, ndx: nat;
	begin
		nNew(n2x);
		nNew(ndx);
		nAdd(n1, nm, ndx);
		nCopy(n2, n2x);
		nMod(n2x, nm);
		nSub(ndx, n2x, ndx);
		nMod(ndx, nm);
		nCopy(ndx, nd);
		nDispose(n2x);
		nDispose(ndx)
	end;



procedure nMulMod (n1, n2, nm, nd: nat);   { (n1 * n2) mod nm ---> nd }
	var
		ndx: nat;
	begin
		nNew(ndx);
		nMul(n1, n2, ndx);
		nMod(ndx, nm);
		nCopy(ndx, nd);
		nDispose(ndx)
	end;



procedure nPowerMod (ns, ne, nm, nd: nat);   { (ns^ne) mod nm ---> nd }
	var
		n, nx, nss: nat;
		pe: Ptr;
		lm: longint;
		size, l, k, i: longint;
	procedure nMulM (n1, n2, nm, nd: nat);
		var
			p1, p2, pm, pd: Ptr;
			ldummy: longint;
		begin
			p1 := Ptr(longint(n1^) + 4);
			p2 := Ptr(longint(n2^) + 4);
			pm := Ptr(longint(nm^) + 4);
			pd := Ptr(longint(nd^) + 4);
			aMul(p1, n1^^.len, p2, n2^^.len, pd, nd^^.len);
			aDiv(pd, nd^^.len, pm, nm^^.len, nil, ldummy)
		end;
	begin
		if nUndefined(ns) or nUndefined(ne) or (nLen(nm) <= 0) then
			nUndefine(nd)
		else if ne^^.len = 0 then
			if ns^^.len = 0 then
				nUndefine(nd)
			else
				long2nat(1, nd)
		else
			begin
				size := nm^^.len * 2 + 1;
				nNewAtLeast(n, size);
				nNewAtLeast(nx, size);
				nNewAtLeast(nss, size);
				nCopy(ns, nss);
				nMod(nss, nm);
				pe := Ptr(longint(ne^) + 4);
				k := ne^^.len * 32 - 1;
				while not aBitTst(pe, k) do
					k := k - 1;
				nCopy(nss, n);
				for i := k - 1 downto 0 do
					begin
						nMulM(n, n, nm, nx);
						if aBitTst(pe, i) then
							nMulM(nx, nss, nm, n)
						else
							nCopy(nx, n)
					end;
				nCopy(n, nd);
				nDispose(n);
				nDispose(nx);
				nDispose(nss);
			end
	end;



procedure nInvMod (ns, nm, nd: nat);
	var
		na1, na2, nb1, nb2, nq, nt, n: nat;
	begin
		if nUndefined(ns) or nUndefined(nm) then
			nUndefine(nd)
		else
			begin
				nNew(na1);
				nNew(na2);
				nNew(nb1);
				nNew(nb2);
				nNew(nq);
				nNew(nt);
				long2nat(1, na1);
				nCopy(ns, na2);   { ns * na1 = na2 (mod nm) }
				long2nat(0, nb1);
				nCopy(nm, nb2);   { ns * nb1 = nb2 (mod nm) }
				while nLen(nb2) > 0 do
					begin
						nDiv(na2, nb2, nq, na2);
						n := na2;
						na2 := nb2;
						nb2 := n;
						nMulMod(nb1, nq, nm, nt);
						nSubMod(na1, nt, nm, na1);
						n := na1;
						na1 := nb1;
						nb1 := n;
					end;
				if nLongCompare(na2, 1) = eq then
					nCopy(na1, nd)
				else
					nUndefine(nd);
				nDispose(na1);
				nDispose(na2);
				nDispose(nb1);
				nDispose(nb2);
				nDispose(nq);
				nDispose(nt);
			end
	end;



procedure nGcd (n1, n2, nd: nat);
	var
		nn1, nn2, nn: nat;
	begin
		if nUndefined(n1) or nUndefined(n2) then
			nUndefine(nd)
		else
			begin
				nNew(nn1);
				nCopy(n1, nn1);
				nNew(nn2);
				nCopy(n2, nn2);
				while nn2^^.len > 0 do
					begin
						nMod(nn1, nn2);
						nn := nn1;
						nn1 := nn2;
						nn2 := nn
					end;
				nCopy(nn1, nd);
				nDispose(nn1);
				nDispose(nn2)
			end
	end;




procedure str2nat (s: string; n: nat);
	var
		radix: integer;
		range: charset;
		limit, bigrad, l: long;
	begin
		radix := strGetRadix(s);
		case radix of
			10: 
				begin
					range := numeral;
					limit := 214748364  { = 2^31 div 10 }
				end;
			16: 
				begin
					range := numeral + ['A'..'F', 'a'..'f'];
					limit := $7ffffff  { = 2^31 div 16 }
				end
		end;
		long2nat(0, n);
		bigrad := 1;
		l := 0;
		while strCIn(s, 1, range) do
			if bigrad < limit then
				begin
					bigrad := bigrad * radix;
					l := l * radix + figure2digit(strGetC(s))
				end
			else
				begin
					nLongMul(n, bigrad, n);
					nLongAdd(n, l, n);
					bigrad := 1;
					l := 0;
				end;
		if bigrad <> 1 then
			begin
				nLongMul(n, bigrad, n);
				nLongAdd(n, l, n);
			end
	end;



function nat2str (n: nat): string;
	const
		tenTo9th = 1000000000;
	var
		s: string;
		temp: nat;
		r: long;
		l, i: longint;
	begin
		l := nLen(n);
		if l < 0 then
			nat2str := 'natNAN'
		else if l = 0 then
			nat2str := '0'
		else if l < 25 then
			begin
				nNew(temp);
				nCopy(n, temp);
				s := '';
				nLongDiv(temp, tenTo9th, temp, r);
				while temp^^.len > 0 do
					begin
						s := concat(strStuffL(long2str(r), '0', 9), s);
						nLongDiv(temp, tenTo9th, temp, r);
					end;
				s := concat(long2str(r), s);
				nDispose(temp);
				nat2str := s
			end
		else
			begin
				s := '$';
				for i := l - 1 downto l - 3 do
					s := concat(s, ' ', long2hex(n^^.val[i]));
				s := concat(s, ' ...');
				for i := 2 downto 0 do
					s := concat(s, ' ', long2hex(n^^.val[i]));
				s := concat(s, ' (len=', long2str(l), ')');
				nat2str := s
			end
	end;





procedure readnat (n: nat);
	var
		l, i: longint;
		s: string;
	begin
		write('*** readnat ***   len = ');
		readln(l);
		nSetLen(n, l);
		write('Input digits : ');
		readln(s);
		for i := 0 to l - 1 do
			begin
				nSetVal(n, i, str2long(strGetW(s, all - [' '])));
				strSkip(s, [' '])
			end
	end;



procedure writenat (n: nat);
	var
		i: longint;
	begin
		write('len(', n^^.len : 1, ')');
		for i := 0 to nSize(n) - 1 do
			write(' ', long2hex(nVal(n, i)));
		writeln
	end;



end.