unit picture;   { June 13, 1991 programmed by Masaaki Wada }

interface

var
	thePicture: PicHandle;

procedure openPic (width, height: integer);
procedure closePic;
procedure showPic;
procedure writePic (name: string);
procedure scrapPic;
procedure killPic;

procedure setScale (xmin, xmax, ymin, ymax: extended);
procedure real2int (x, y: extended; var i, j: integer);
procedure int2real (i, j: integer; var x, y: extended);
procedure realMoveTo (x, y: extended);
procedure realLineTo (x, y: extended);
procedure setWidth (w: extended);
procedure realLine (x1, y1, x2, y2: extended);
procedure realBezier (x1, y1, x2, y2, x3, y3, x4, y4: extended);

implementation

var
	tempPort: CGrafPtr;
	savePort, window: WindowPtr;
	xscale, yscale, xmn, ymx: extended;
	width: extended;

procedure openPic (width, height: integer);
	var
		r: Rect;
	begin
		GetPort(savePort);
		tempPort := CGrafPtr(NewPtr(SizeOf(CGrafPort)));
		OpenCPort(tempPort);
		SetPort(GrafPtr(tempPort));
		SetRect(r, 0, 0, width, height);
		ClipRect(r);
		thePicture := OpenPicture(r);
	end;

procedure closePic;
	begin
		ClosePicture;
		SetPort(savePort);
		CloseCPort(tempPort);
	end;

procedure showPic;
	var
		hratio, vratio, ratio: extended;
		h, v: integer;
		frame, r: Rect;
	begin
		frame := thePicture^^.picFrame;
		with screenbits.bounds do
			begin
				hratio := (right - left - 10) / frame.right;
				vratio := (bottom - top - 50) / frame.bottom;
			end;
		if hratio < vratio then
			ratio := hratio
		else
			ratio := vratio;
		h := trunc(ratio * frame.right);
		v := trunc(ratio * frame.bottom);
		SetRect(r, 5, 45, 5 + h, 45 + v);

		GetPort(savePort);
		window := NewCWindow(nil, r, 'Draw', true, noGrowDocProc, WindowPtr(-1), false, 0);
		SetPort(window);

		SetRect(r, 0, 0, h, v);
		DrawPicture(thePicture, r);
		SetPort(savePort);
	end;

procedure writePic (name: string);
	var
		p: Point;
		reply: SFReply;
		refnum, err, i: integer;
		l, longZero: longint;
	begin
		SetPt(p, 100, 80);
		SFPutFile(p, 'Save the PICT as:', name, nil, reply);
		with reply do
			if good then
				begin
					err := Create(fName, vRefNum, 'ttxt', 'PICT');
					if (err = noErr) or (err = dupFNErr) then
						begin
							err := FSOpen(fName, vRefNum, refnum);
							longZero := 0;
							l := 4;
							for i := 1 to 512 div 4 do
								err := FSWrite(refnum, l, @longZero);
							l := GetHandleSize(Handle(thePicture));
							err := FSWrite(refnum, l, Ptr(thePicture^));
							err := FSClose(refnum);
						end
				end;
	end;

procedure scrapPic;
	var
		dummy, len: longint;
	begin
		dummy := ZeroScrap;
		len := GetHandleSize(Handle(thePicture));
		dummy := PutScrap(len, 'PICT', Ptr(thePicture^))
	end;

procedure killPic;
	begin
		KillPicture(thePicture)
	end;

procedure setScale (xmin, xmax, ymin, ymax: extended);
	var
		r: rect;
	begin
		xmn := xmin;
		ymx := ymax;
		r := thePicture^^.picFrame;
		xscale := (xmax - xmin) / r.right;
		yscale := (ymax - ymin) / r.bottom
	end;

procedure real2int (x, y: extended; var i, j: integer);
	begin
		i := round((x - xmn) / xscale);
		j := round((ymx - y) / yscale)
	end;

procedure int2real (i, j: integer; var x, y: extended);
	begin
		x := xmn + i * xscale;
		y := ymx - j * yscale
	end;

procedure realMoveTo (x, y: extended);
	var
		i, j: integer;
	begin
		real2int(x, y, i, j);
		MoveTo(i, j)
	end;

procedure realLineTo (x, y: extended);
	var
		i, j: integer;
	begin
		real2int(x, y, i, j);
		LineTo(i, j)
	end;


procedure setWidth (w: extended);
	begin
		width := w
	end;


procedure realLine (x1, y1, x2, y2: extended);
	var
		x, y, r, u, v: extended;
		p: PolyHandle;
	begin
		x := x2 - x1;
		y := y2 - y1;
		r := sqrt(sqr(x) + sqr(y));
		if r > 0 then
			begin
				u := -y / r * 0.5 * width;
				v := x / r * 0.5 * width;
				p := OpenPoly;
				realMoveTo(x1 + u, y1 + v);
				realLineTo(x1 - u, y1 - v);
				realLineTo(x2 - u, y2 - v);
				realLineTo(x2 + u, y2 + v);
				realLineTo(x1 + u, y1 + v);
				ClosePoly;
				PaintPoly(p);
				KillPoly(p)
			end
	end;


procedure realBezier (x1, y1, x2, y2, x3, y3, x4, y4: extended);
	var
		r, rmx, t: extended;
		e12, f12, e23, f23, e34, f34: extended;
		u1, v1, u2, v2, u3, v3, u4, v4: extended;
		x12, y12, x23, y23, x34, y34: extended;
		x123, y123, x234, y234, x1234, y1234: extended;
		p: PolyHandle;
	procedure midpoint (x1, y1, x2, y2: extended; var x, y: extended);
		begin
			x := 0.5 * (x1 + x2);
			y := 0.5 * (y1 + y2)
		end;
	begin
		r := sqrt(sqr(x2 - x1) + sqr(y2 - y1));
		if r <= 0 then
			exit(realBezier);
		rmx := r;
		e12 := -(y2 - y1) / r;
		f12 := (x2 - x1) / r;

		r := sqrt(sqr(x3 - x2) + sqr(y3 - y2));
		if r <= 0 then
			exit(realBezier);
		if r > rmx then
			rmx := r;
		e23 := -(y3 - y2) / r;
		f23 := (x3 - x2) / r;

		r := sqrt(sqr(x4 - x3) + sqr(y4 - y3));
		if r <= 0 then
			exit(realBezier);
		if r > rmx then
			rmx := r;
		e34 := -(y4 - y3) / r;
		f34 := (x4 - x3) / r;

		if r <= 0.2 * width then
			begin
				u1 := 0.5 * width * e12;
				v1 := 0.5 * width * f12;

				u2 := e12 + e23;
				v2 := f12 + f23;
				t := 0.5 * width / (u2 * e23 + v2 * f23);
				u2 := t * u2;
				v2 := t * v2;

				u3 := e23 + e34;
				v3 := f23 + f34;
				t := 0.5 * width / (u3 * e23 + v3 * f23);
				u3 := t * u3;
				v3 := t * v3;

				u4 := 0.5 * width * e34;
				v4 := 0.5 * width * f34;

				p := OpenPoly;
				realMoveTo(x1 + u1, y1 + v1);
				realLineTo(x1 - u1, y1 - v1);
				realLineTo(x2 - u2, y2 - v2);
				realLineTo(x3 - u3, y3 - v3);
				realLineTo(x4 - u4, y4 - v4);
				realLineTo(x4 + u4, y4 + v4);
				realLineTo(x3 + u3, y3 + v3);
				realLineTo(x2 + u2, y2 + v2);
				realLineTo(x1 + u1, y1 + v1);
				ClosePoly;
				PaintPoly(p);
				KillPoly(p)
			end
		else
			begin
				midpoint(x1, y1, x2, y2, x12, y12);
				midpoint(x2, y2, x3, y3, x23, y23);
				midpoint(x3, y3, x4, y4, x34, y34);
				midpoint(x12, y12, x23, y23, x123, y123);
				midpoint(x23, y23, x34, y34, x234, y234);
				midpoint(x123, y123, x234, y234, x1234, y1234);
				realBezier(x1, y1, x12, y12, x123, y123, x1234, y1234);
				realBezier(x1234, y1234, x234, y234, x34, y34, x4, y4)
			end
	end;




end.